# Generated by roxygen2: do not edit by hand

S3method(plot,batss)
S3method(print,batss)
S3method(print,summary.batss)
S3method(summary,batss)
export(RAR.optimal)
export(RAR.trippa)
export(alloc.balanced)
export(alloc.simple)
export(batss.combine)
export(batss.glm)
export(eff.arm.infofract)
export(eff.arm.simple)
export(eff.trial.all)
export(eff.trial.any)
export(fut.arm.simple)
export(fut.trial.all)
import(R.utils)
import(abind)
import(grDevices)
import(graphics)
import(methods)
import(parallel)
import(plyr)
import(stats)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_h3)
importFrom(grDevices,dev.new)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,gray)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,matpoints)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,is)
importFrom(plyr,is.formula)
importFrom(rlang,dots_list)
importFrom(stats,dnorm)
importFrom(stats,pnorm)
importFrom(stats,qbinom)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,packageVersion)
