#' BATSS
#'
#' @name BATSS
#' @import plyr
#' @import R.utils
#' @import parallel
#' @import methods
#' @import stats
#' @import graphics
#' @import grDevices
#' @import abind
#' @importFrom graphics axis legend lines matplot matpoints mtext points abline layout par text title
#' @importFrom stats dnorm pnorm qnorm rnorm uniroot qbinom quantile var
#' @importFrom grDevices dev.new devAskNewPage gray
#' @importFrom methods is
#' @importFrom rlang dots_list
#' @importFrom plyr is.formula
#' @importFrom cli cli_h1 cli_h2 cli_h3
#' @importFrom utils packageVersion
NULL
